/* line.c
 */

#include "statistics.h"

#include <stdio.h>

double
compute_line(unsigned n, const double *x, const double *y,
	     double *m, double *e)
{
  unsigned i;

  double x_sum = 0.0;
  double y_sum = 0.0;
  double xx_sum = 0.0;
#if 0
  double yy_sum = 0.0;
#endif
  double xy_sum = 0.0;

  double x_var;
  double xy_covar;
  
  double k;

  if (n<2)
    {
      if (m)
	*m = n ? y[0] : 0.0;
      if (e)
	*e = 0.0;
      return 0.0;
    }
  
  for (i = 0; i<n; i++)
    {
      x_sum += x[i];
      y_sum += y[i];
      xx_sum += (x[i] * x[i]);
#if 0
      yy_sum += (y[i] * y[i]);
#endif      
      xy_sum += (x[i] * y[i]);
    }

  /* Find y* = kx + mz that minimizes |y - y*|, where z = (1, ..., 1).
   * Change to an ON-basis by setting
   *
   *   x* = x - mean(x) z,
   *
   * y* = k x* + m* z = k x + (m* - mean(x)) z
   *
   * Then k = (y, x*)/(x*,x*), m* = (y, z)/(z,z), and
   * 
   * m = m* - mean(x).
   *
   * We also get the residual vector of length
   *
   * |y|^2 - k^2 |x*|^2 - m*^2|z|^2
   */

#if 0
  fprintf(stderr, "x_sum = %f, xx_sum = %f, y_sum = %f, xy_sum = %f\n",
	  x_sum, xx_sum, y_sum, xy_sum);
#endif
  x_var = (xx_sum - x_sum * x_sum / n);
  xy_covar = (xy_sum - x_sum * y_sum / n);
  
  k = xy_covar / x_var;

#if 0
  fprintf(stderr, "x_var = %f, xy_covar = %f, k = %f\n",
	  x_var, xy_covar, k);

  fprintf(stderr, "m = %f\n", (y_sum - k * x_sum)/n);
#endif
  
  if (m)
    *m = (y_sum - k * x_sum)/n;

  if (e)
    *e = /* yy_sum */ - (xy_covar*xy_covar / x_var + y_sum*y_sum / n);

  return k;
}
